@@ -190,7 +190,7 @@ class HeaderBuilder(SectionBuilder):
         return [f'{self._root.get_help_naming_string()} - {self.get_header_description_string()}']
 
     def get_header_description_string(self) -> str:
-        return self._root.help.short_description
+        return self._root.get_short_description()
 
 
 class SynopsisBuilder(SectionBuilder):
@@ -199,14 +199,14 @@ class SynopsisBuilder(SectionBuilder):
         return 'Synopsis'
 
     def _build_section(self):
-        return [self._root.help.synopsis or self._build_synopsis()]  # TODO: implement
+        return [self._root.get_synopsis() or self._build_synopsis()]  # TODO: implement
 
     def _build_synopsis(self) -> str:
         return ''
 
     def _bracket(self, to_bracket: i_help_type) -> str:
         if isinstance(to_bracket, FinalNode):
-            return f'<{to_bracket.help.name}>' if to_bracket.has_lower_limit() else f'[{to_bracket.help.name}]'
+            return f'<{to_bracket.get_help_naming_string()}>' if to_bracket.has_lower_limit() else f'[{to_bracket.get_help_naming_string()}]'
 
 
 class DescriptionBuilder(SectionBuilder):
@@ -215,7 +215,7 @@ class DescriptionBuilder(SectionBuilder):
         return 'Description'
 
     def _build_section(self):
-        return self._root.help.long_description
+        return self._root.get_long_description()
 
 
 class SubHelpBuilder(SectionBuilder, ABC):
@@ -224,9 +224,10 @@ class SubHelpBuilder(SectionBuilder, ABC):
         sub_helps = self._root.get_sub_helps()
         name = self.get_section_name()
         kind = HelpType(name)
-        if kind in sub_helps:
+        try:
             return sub_helps[kind]
-        return []
+        except KeyError:
+            return []
 
     def _build_section(self):
         return reduce(op.add, map(self.build_single_sub_help, self.get_sub_helps()), [])
@@ -235,7 +236,7 @@ class SubHelpBuilder(SectionBuilder, ABC):
         return [sub_help.get_help_naming_string(), self.build_single_sub_help_description(sub_help)]
 
     def build_single_sub_help_description(self, sub_help: IHelp) -> list[str] | str:
-        return [sub_help.help.short_description]
+        return [sub_help.get_short_description()]
 
 
 class ParametersSectionBuilder(SubHelpBuilder):
@@ -254,7 +255,7 @@ class HiddenNodesSectionBuilder(SubHelpBuilder):
 
     def build_single_sub_help(self, sub_help: IHelp) -> list:
         built = super().build_single_sub_help(sub_help)
-        built += [[sub_help.help.synopsis]]
+        built += [[sub_help.get_synopsis()]]
         return built
 
 
@@ -279,6 +280,10 @@ class IHelp(ABC):
     Interface for objects that have help and can be managed by HelpManager
     '''
 
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self._long_description: list[str, ...] = []
+
     @abstractmethod
     def get_help(self) -> Help:
         raise NotImplementedError
@@ -295,18 +300,27 @@ class IHelp(ABC):
         raise NameError
 
     def get_help_naming_string(self):
-        naming = self.help.name or self._get_help_naming()
+        naming = self._get_help_naming()
         if not isinstance(naming, str):
             naming = str(list(naming))[1:-2]
         return naming
 
+    def get_short_description(self) -> str:
+        return self.help.short_description if self.help.short_description is not None else ''
+
+    def get_long_description(self) -> str:
+        return self.help.long_description if self.help.long_description is not None else ''
+
+    def get_synopsis(self) -> str:
+        return self.help.synopsis if self.help.synopsis is not None else ''
+
 
 @dataclass
 class Help:
-    name: str = ''
-    short_description: str = ''
-    long_description: str = ''
-    synopsis: str = ''
+    short_description: str = None
+    long_description: str = None
+    synopsis: str = None
+
 
 ###################
 # Default storage #
@@ -352,18 +366,12 @@ class DefaultStorage(IDefaultStorable):
         to_return = next((get_default() for condition, get_default in reversed(self._get_defaults.items()) if condition()), None)
         return to_return
 
-    def __contains__(self, item):
-        if not isinstance(item, (int, float, str, list, dict, set)) and 'name' in item.__dict__:
-            item = item.name
-
-        return super().__contains__(item)
-
 ###########################
 # Activations and actions #
 ###########################
 
 
-class IActivable(ABC):
+class IActivable(INamable, ABC):
 
     @staticmethod
     def _map_to_single(*to_map: compositeActive, func: bool_from_iterable = all) -> bool_from_void | None:
@@ -420,7 +428,7 @@ class ImplicitlyActivableMixin(IActivable):
         return self._activated
 
 
-class ConditionallyActiveMixin(IActivable):
+class ConditionallyActiveMixin(IActivable, IHelp, ABC):
 
     def __init__(self, active_condition: compositeActive = None, inactive_condition: compositeActive = None, default_state: bool = None, **kwargs):
         super().__init__(**kwargs)
@@ -491,7 +499,7 @@ class ConditionallyActiveMixin(IActivable):
         self.set_inactive_on_conditions(lambda: not len(collection))
 
 
-class ActionOnActivationMixin:
+class ActionOnActivationMixin(INamable, IHelp, ABC):
     T = TypeVar('T')
 
     def __init__(self, **kwargs):
@@ -508,6 +516,7 @@ class ActionOnActivationMixin:
     def when_active_set_activated(self, activated: bool, *to_set: IActivable):
         activate_once = lambda a: a.set_activated(activated)
         self.when_active_apply_for_all(activate_once, to_set)
+        self._long_description.append(f'When {self.name} is activated, it turns {"on" if activated else "off"} {", ".join(map(INamable.get_name, to_set))}')
 
     def when_active_apply_for_all(self, func: Callable[[T], Any], elems: Iterable[T]):
         self.when_active(lambda: (func(elem) for elem in elems))
@@ -539,12 +548,14 @@ class ActionOnActivationMixin:
             func()
 
     def _perform_additional_actions_on_activation(self) -> None:
-        for cond, action in self._additional_actions.items():
-            if cond():
-                action()
+        for action in self._get_active_additional_actions():
+            action()
+
+    def _get_active_additional_actions(self) -> Iterable[any_from_void]:
+        return (action for action, cond in self._additional_actions.items() if cond())
 
 
-class ActionOnImplicitActivation(ImplicitlyActivableMixin, ActionOnActivationMixin):
+class ActionOnImplicitActivation(ImplicitlyActivableMixin, ActionOnActivationMixin, ABC):
 
     def __init__(self, activated=False, **kwargs):
         super().__init__(activated=activated, **kwargs)
@@ -555,7 +566,7 @@ class ActionOnImplicitActivation(ImplicitlyActivableMixin, ActionOnActivationMix
         self._perform_additional_actions_on_activation()
 
 
-class ActionOnCondition(ConditionallyActiveMixin, ActionOnActivationMixin):
+class ActionOnCondition(ConditionallyActiveMixin, ActionOnActivationMixin, ABC):
 
     def __init__(self, active_condition: compositeActive = None, inactive_condition: compositeActive = None, **kwargs):
         super().__init__(active_condition=active_condition, inactive_condition=inactive_condition, **kwargs)
@@ -592,14 +603,17 @@ class FlagManagerMixin:
     def __getitem__(self, name: str):
         return self.get_flag(name)
 
-    def get_flag(self, name: str):
+    def get_flag(self, name: str) -> Flag:
         return next((flag for flag in self._flags if flag.has_name(name)))
 
     def get_flags(self, *flag_names: str) -> list[Flag]:
         if not flag_names:
             return self._flags
         else:
-            return [flag for flag in self._flags if any(name in flag_names for name in flag.get_all_names())]
+            return [flag for flag in self._flags if flag.has_name_in(flag_names)]
+
+    def get_active_flags(self) -> Iterable[Flag]:
+        return filter(Flag.is_active, self._flags)
 
     def add_flag(self, main: str | Flag, *alternative_names: str, storage: CliCollection = None, storage_limit: int | None = -1, storage_lower_limit=-1, default: default_type = None, flag_limit=-1, flag_lower_limit=-1) -> Flag:
         name = get_name(main)
@@ -627,10 +641,13 @@ class FlagManagerMixin:
 
     def _chunk_by_flags(self, args: list[str]) -> Iterator[list[str]]:
         curr_i = 0
+        met_node: VisibleNode | None = None
         for i, arg in enumerate(args):
-            if self.has_flag(arg):
+            if self.has_flag(arg) and not (met_node and met_node.has_flag(arg)):
                 yield args[curr_i: i]
                 curr_i = i
+            elif self.has_visible_node(arg):  # TODO: Flag mixin does not have those method - think of refactoring it
+                met_node = self.get_visible_node(arg)
         yield args[curr_i:]
 
     def _filter_flags_out_of_chunk(self, chunk: list[str], activate=True) -> list[str]:
@@ -749,6 +766,8 @@ class ParameterManagerMixin(IResetable):
         if right is None:
             right = self._find_greater_arity_for_arity(arity, allowed)
 
+        if not right:
+            raise IncorrectArity(arity, '~' + str(allowed))
         return self._orders[right]
 
     def get_allowed_arities(self) -> Iterable[int]:
@@ -768,13 +787,14 @@ class ParameterManagerMixin(IResetable):
         return true_minimal_arity <= arity_to_check
 
     # lt - less than
-    def _find_multi_param_lt_arity_for_arity(self, arity: int, allowed_arities: list[int]) -> int:
-        multi_param_arities = filter(lambda a: self.get_param(self._orders[a][-1]).is_multi(), allowed_arities)
+    def _find_multi_param_lt_arity_for_arity(self, arity: int, allowed_arities: list[int]) -> int | None:
+        with_params = filter(lambda a: bool(self._orders[a]), allowed_arities)
+        multi_param_arities = filter(lambda a: self.get_param(self._orders[a][-1]).is_multi(), with_params)
         smaller_arities = filter(lambda a: a < arity, multi_param_arities)
         return max(smaller_arities, default=None)
 
-    def _find_greater_arity_for_arity(self, arity: int, allowed_arities: list[int]):
-        return min(filter(lambda a: a > arity, allowed_arities))
+    def _find_greater_arity_for_arity(self, arity: int, allowed_arities: list[int]) -> int | None:
+        return min(filter(lambda a: a > arity, allowed_arities), default=None)
 
     def _get_param_names_to_skip_for(self, order: list[str], arity: int) -> Iterable[str]:
         must_be_skipped = list(self._get_param_names_that_must_be_skipped(order))
@@ -830,7 +850,7 @@ class ParameterManagerMixin(IResetable):
 
 class HiddenNodeManagerMixin:
     def __init__(self, **kwargs):
-        super().__init__()
+        super().__init__(**kwargs)
         self._hidden_nodes: dict[str, HiddenNode] = {}
 
     def add_hidden_node(self, to_add: str | Node, active_condition: Callable[[], bool] = None, action: Callable = None) -> HiddenNode:
@@ -877,16 +897,17 @@ class HiddenNodeManagerMixin:
 #############
 
 
-class Node(INamable, IHelp, ParameterManagerMixin, IResetable, ActionOnActivationMixin, FlagManagerMixin, HiddenNodeManagerMixin):
+class Node(ParameterManagerMixin, IResetable, ActionOnActivationMixin, FlagManagerMixin, HiddenNodeManagerMixin, INamable, IHelp):
 
     def __init__(self, name: str, parameters: Iterable[str | Parameter] = None, param_storages: tuple[CliCollection] = (),
                  short_description: str = '', long_description: str = '', **kwargs):
-        super().__init__(name=name, parameters=parameters, param_storages=param_storages, **kwargs)
+        super().__init__(name=name, parameters=parameters, storages=param_storages, **kwargs)
         self._visible_nodes: dict[str, VisibleNode] = dict()
         self._collections: dict[str, CliCollection] = dict()
         self._actions: dict[bool_from_void, SmartList[any_from_void]] = dict()
         self._action_results: list = []
         self._only_hidden = False
+        self._help_manager = HelpManager(self)
         self._help = Help(short_description, long_description)
 
     # Help
@@ -905,6 +926,19 @@ class Node(INamable, IHelp, ParameterManagerMixin, IResetable, ActionOnActivatio
     def _get_help_naming(self) -> Iterable[str] | str:
         return self.get_name()
 
+    def add_general_help_flag_to_all(self, main: str, *alternative_names: str, action: any_from_void = None) -> None:
+        to_apply = lambda node: node._add_general_flag_to_self(main, *alternative_names, action=action)
+        self.apply_to_self_and_all_nodes(to_apply)
+
+    def _add_general_flag_to_self(self, main: str, *alternative_names: str, action: any_from_void) -> None:
+        flag = self.add_flag(main, *alternative_names)
+        action = action or (lambda: self._help_manager.print_help())
+        self.add_action_when_is_active(action, flag)
+
+    @property
+    def help_manager(self) -> HelpManager:
+        return self._help_manager
+
     # Resetable
 
     def reset(self) -> None:
@@ -957,6 +991,11 @@ class Node(INamable, IHelp, ParameterManagerMixin, IResetable, ActionOnActivatio
         except LookupError:
             return False
 
+    def apply_to_self_and_all_nodes(self, to_apply: Callable, **kwargs):
+        to_apply(self)
+        for node in self.get_all_nodes():
+            node.apply_to_self_and_all_nodes(to_apply, **kwargs)
+
     # Nodes
     def get_node(self, name: str) -> Node:
         return next((nodes[name] for nodes in [self._visible_nodes, self._hidden_nodes] if name in nodes))
@@ -1155,7 +1194,6 @@ class CliCollection(DefaultStorage, SmartList, INamable, IResetable):
     def get_lower_limit(self) -> int:
         return self._lower_limit
 
-
     def get_nth(self, n: int):
         return self.get_plain()[n]
 
@@ -1164,7 +1202,7 @@ class CliCollection(DefaultStorage, SmartList, INamable, IResetable):
         :return: As get plain but if the collection has length of 1 gets the only element of it
         '''
         to_return = self.get_plain()
-        if isinstance(to_return, Sized) and isinstance(to_return, Iterable) and len(to_return) == 1:
+        if isinstance(to_return, Sized) and isinstance(to_return, Iterable) and len(to_return) == 1 and not isinstance(to_return, str):
             to_return = next(iter(to_return))
         return to_return
 
@@ -1198,8 +1236,15 @@ class CliCollection(DefaultStorage, SmartList, INamable, IResetable):
 
     def __contains__(self, item):
         if isinstance(item, Flag):
-            return any(name in self.get_plain() for name in item.get_all_names())
-        return super().__contains__(item)
+            items = self.get_as_list()
+            names = filter(lambda elem: isinstance(elem, str), items)
+            flags = filter(lambda elem: isinstance(elem, Flag), items)
+            return item.has_name_in(names) or any(item.has_name_in(flag.get_all_names()) for flag in flags)
+
+        if isinstance(item, INamable):
+            item = item.name
+
+        return item in self.get_as_list()
 
     def __hash__(self):
         return hash(tuple(self))
@@ -1377,7 +1422,7 @@ class FinalNode(IDefaultStorable, INamable, IResetable, IHelp, ABC):
         '''
         :return: Return truncated to the limit values of the collection or if there are no values, returns the default values
         '''
-        to_return = self._storage.get_plain()
+        to_return = self._storage.get_as_list()
         if isinstance(to_return, Sized):
             if self._limit is not None and self._limit < len(to_return):
                 to_return = to_return[:self._limit]
@@ -1424,11 +1469,19 @@ class Flag(FinalNode, ActionOnImplicitActivation):
     def _get_help_naming(self) -> Iterable[str] | str:
         return self.get_all_names()
 
+    def get_long_description(self) -> str:
+        if self.help.long_description is not None:
+            return super().get_long_description()
+        raise NotImplementedError
+
+
     # Reset
 
     def reset(self):
         self.deactivate()
 
+    # Common
+
     def add_alternative_names(self, *alternative_names: str):
         self._alternative_names |= set(alternative_names)
 
@@ -1441,6 +1494,9 @@ class Flag(FinalNode, ActionOnImplicitActivation):
     def get_all_names(self) -> list[str]:
         return [self._name] + list(self._alternative_names)
 
+    def __hash__(self):
+        return hash((self.name, *self._alternative_names))
+
 
 default_type = str | int | list[str | int] | None
 i_help_type = Node | Flag | Parameter | HiddenNode | VisibleNode
